/***************************************************************************
                          syntaxhighlighter.h  -  description
                             -------------------
    begin                : thu apr 17 2003
    copyright            : (C) 2003 by Piotr Mierzwiski
    email                : peterm@go2.pl

    Copyright: See COPYING file that comes with this project
***************************************************************************/

#ifndef _SYNTAXHIGHLIGHTER_H_
#define _SYNTAXHIGHLIGHTER_H_

#include <qsyntaxhighlighter.h>
#include <qtextedit.h>

/**
 * Author:
 * Piotr Mierzwiski
 **/
/** @short Klasa zarzdza kolorowaniem skadni widoku tekstowego.
 Klaas ma za zadanie wykonanie kolorowania skadni dla podanego jego
 rodzaju pliku. Aby mc wykorzysta klase naley j powiza z obiektem klasy
 QTextEdit, do krego naley poda wskanik w konstruktorze. \n
 Reguy kolorowania s wczytywane z pliku konfiguracyjnego
 (jeli nie zostan tam znalezione, wtedy brane s domylne).\n
 Obugiwane jest kolorowanie skadni dla jzykw: C/C++ oraz HTML/XML.
*/
class SyntaxHighlighter : public QSyntaxHighlighter
{
public:
	/** Lista rodzajw kolorowania.
	*/
	enum SyntaxHighlighting {
		NONE=0,
		HTML=100, CSS, LaTeX, XML, SGML, VRML, // markup
		BASH=200, PERL, PHP, PYTHON, AWK, SQL, RUBY, // scripts
		CCpp=300, JAVA, ASM, PASCAL, EIFFEL, PROLOG, HASKELL, ADA, // source
		HEX=400,  CHANGELOG, DIFF, MAKEFILE, RPMspec, // other
		AUTO=1000
	};

	/** Konstruktor klasy, zajmujcej si "podwietlaniem" skadni.\n
	 @param textEdit - wskanik na obiekt podgldu tekstowego.\n
	 Inicjowane s tutaj: domylna czcionka oraz regy kolorowania skadni.
	*/
	SyntaxHighlighter( QTextEdit * textEdit );

	/** Inicjuje kolorowanie skadni.\n
	 @param sh - rodzaj kolorowania skadni, patrz @see SyntaxHighlighting,
	 @param ext - rozszerzenie pliku, dla ktrego nastpuje kolorowanie skadni.\n
	 Ustawiana jest tutaj skadowa okrelajca rodzaj kolorowania skadni
	 oraz wczytywane z pliku konfiguracyjnego reguy kolorowania skadni dla
	 wybranego wczeniej jego rodzaju.
	 */
	void setSyntaxHighlighting( SyntaxHighlighting sh, const QString & ext=QString::null );

	/** Inicjuje podanymi wartociami biecy kontekst kolorowania.\n
	 @param context - kontekst kolorowania, patrz: @see ContextOfSH,
	 @param color - kolor w danym kontekcie,
	 @param bold - pogrubienie tekstu (TRUE) w danym kontekcie,
	 @param italic - pochylenie tekstu (TRUE) w danym kontekcie.\n
	 Jest tutaj inicjowana tablica atrybutw, z ktrej s pobierane informacje
	 o sposobie kolorowania w funkcji ustawiajcej atrybuty dla kontekstu.
	 */
	void initAttributsTable( int context, const QColor & color, bool bold, bool italic );

	/** Zwraca biecy rodzaj kolorowania skadni.\n
	 @em return - rodzaj kolorowania skadni
	 */
	SyntaxHighlighting currentHighlighting() const { return mSyntaxHighlighting; }

private:
	SyntaxHighlighting mSyntaxHighlighting;
	QColor mColor;
	QFont mFont;

	QStringList mCcppRullesList, mHtmlRullesList;

	struct Attribut {
		QColor color;
		bool bold;
		bool italic;
	};
	Attribut AttrTab[11];

	enum ContextOfSH {
		BASE_N_INTEGER,
		CHARACTER,
		COMMENT,
		DATA,
		DECIMALorVALUE,
		EXTENSION,
		FLOATING_POINT,
		KEYWORD,
		NORMAL,
		OTHERS,
		STRING
	};

	typedef QMap<QString, ContextOfSH> ContextMap;
	ContextMap mContextMap;

protected:
	/** Funkcja wywoywana automatycznie przy kadej zmianie zawartoci obiektu
	 rodzica (QTextEdit).\n
	 @param text - aktualnie zmodyfikowana linia (cig zakoczony znakiem '\n').
	 @param endStateOfLastPara - status zakoczenia kolorowania dla biecego
	 paragrafu.
	 @return status zakoczenia kolorowania biecego paragrafu
	 */
	int highlightParagraph( const QString & text, int endStateOfLastPara );


	/** Funkcja parsujca jedn lini tekstu dla rda C/CPP oraz ustawiajca
	 w niej podwietlanie skadni.\n
	 @param text - linia tekstu do przeparsowania (cig zakoczony znakiem '\n'),
	 @param endStateOfLastPara - informuje o wieloliniowym kolorowaniem
	 (wicej informacji szukaj w dokumentacji Qt - klasa QSyntaxHighlighting)
	 @return status zakoczenia kolorowania biecego paragrafu
	 */
	int setSyntaxHighlightingForCCpp( const QString & text, int endStateOfLastPara );

	/** Funkcja parsujca jedn lini tekstu dla rda HTML oraz ustawiajca
	 w niej podwietlanie skadni.\n
	 @param text - linia tekstu do przeparsowania (cig zakoczony znakiem '\n'),
	 @param endStateOfLastPara - informuje o wieloliniowym kolorowaniem
	 (wicej informacji szukaj w dokumentacji Qt - klasa QSyntaxHighlighting)
	 @return status zakoczenia kolorowania biecego paragrafu
	 */
	int setSyntaxHighlightingForHtml( const QString & text, int endStateOfLastPara );

	/** Funkcja ustawiajca w kolorowanie skadni w jednej linii dla widoku HEX.\n
	 @param text - linia tekstu do przeparsowania (cig zakoczony znakiem '\n'),
	 @param endStateOfLastPara - informuje o wieloliniowym kolorowaniem
	 (wicej informacji szukaj w dokumentacji Qt - klasa QSyntaxHighlighting)
	 @return status zakoczenia kolorowania biecego paragrafu
	*/
	int setSyntaxHighlightingForHEX( const QString & text, int endStateOfLastPara );


	/** Funkcja ustawia atrybuty: color, bold, italic (bdce skadowymi klasy)
	 dla podanego kontekstu.\n
	 @param context - kontekst kolorowania.
	 */
	void setAttributsFor( ContextOfSH );


	/** Funkcja wykonuje sprawdzenie czy podany znak ma specjalne znaczenie
	 w poczeniu z '\', np. '\n'. Pomocna przy parsowaniu C/C++.\n
	 @param chr - znak do sprawdzenia,
	 @return TRUE, jeli powyszy warunek zosta speniony, w przeciwnym razie
	 FALSE.\n
	 Jest uywana przez funkcje kolorowania skadni dla C/C++.
	 */
	bool specChar( const QChar & );

	/** Funkcja sprawdza czy podany znak :\n
		@li nie jest du lub ma liter,
		@li jest rny od '$',
		@li jest rwny '\n' lub '\t'.
	 @param chr - znak do sprawdzenia,
	 @return TRUE, jeli powyszy warunek zosta speniony, w przeciwnym razie
	 FALSE.\n
	 Jest uywana przez funkcje kolorowania skadni dla C/C++.
	 */
	bool charOkForDigit( const QChar & chr);

	/** Funkcja sprawdzajca czy podany zosta znak, ktry moe wystpi za sowem
	 kluczowym jzyka C/C++.\n
	 @param chr - znak do sprawdzenia,
	 @return TRUE, jeli powyszy warunek zosta speniony, w przeciwnym razie
	 FALSE.\n
	 Jest uywana przez funkcje kolorowania skadni dla C/C++.
	 */
	bool charOkBehindKeyword( const QChar & chr );

};

#endif
